//----------------------------------------------------------------------------
// (C) 2022 Red Lion Controls, Inc. All rights reserved.
// Red Lion and the Red Lion logo are registered trademarks of Red Lion Controls, Inc.
// All other company and product names are trademarks of their respective owners.
//----------------------------------------------------------------------------
//
//  Module: Redboot led
//
//  File: led.h
//
//  Description:
//  ------------
//  This file contains the macros, structure definitions, and function
//  prototypes used by the RedBoot led command.
//
//----------------------------------------------------------------------------

#include <cyg/io/io.h>                        // Driver interface

#ifndef INCLUDE_RL_LED_H_
#define INCLUDE_RL_LED_H_

#define LED_OFF          0                    // No bits set
#define LED_GREEN        1                    // Green - bit 0 set
#define LED_RED          2                    // Red - bit 1 set
#define LED_YELLOW       3                    // Yellow - bits 0 and 1 set

#define MAX_LEDS        48                    // Maximum number of LEDs on the board

#define TRUE            1
#define FALSE           0

#define LED_YELLOW_CMND_ARG       "-y"        // Command line arg for yellow leds
#define LED_RED_CMND_ARG          "-r"        // Command line arg for red leds
#define LED_GREEN_CMND_ARG        "-g"        // Command line arg for green leds
#define LED_OFF_CMND_ARG          "-n"        // Command line arg for off leds

#define LED_KEY_CLOCK_PIN         0           // Clock Pin config key
#define LED_KEY_DO_PIN            1           // Data Out Pin config key
#define LED_KEY_LD_PIN            2           // Load Data Pin config key
#define LED_KEY_EN_PIN            3           // Load Data Pin config key
#define LED_KEY_OE_RGSTR          4           // Output Register config key
#define LED_KEY_OE_RGSTR_VAL      5           // Output Register config key
#define LED_KEY_SET_RGSTR         6           // GPIO Set Register config key
#define LED_KEY_CLR_RGSTR         7           // GPIO Clear Register config key
#define LED_KEY_OUT_RGSTR         8           // GPIO Output Register config key
#define LED_KEY_OUT_RGSTR_VAL     9           // GPIO Output Register Value config key
#define LED_KEY_LED_CNT           10          // LED Count config key
#define LED_KEY_SHIFTS_0          11          // Least significant bits shifted config key
#define LED_KEY_SHIFTS_1          12          // Middle most significant bits shifted config key
#define LED_KEY_SHIFTS_2          13          // Most significant bits shifted config key

int initializeLedState(int argc,              // Initialize state of one or more LEDs
                       char *argv[],
                       int ndx,
                       unsigned char ledState);

void getDriverCfg(cyg_io_handle_t hndl);      // Gets and prints driver configuration

#endif /* INCLUDE_RL_LED_H_ */
