//==========================================================================
//
//      mscc_simage.c
//
//      Support code/data for MSCC signed image
//
//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
// Copyright (C) 2006 Free Software Foundation, Inc.                        
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Lars Povlsen
// Contributors: 
// Date:         2015-12-08
// Purpose:      
// Description:  
//              
// This code is part of RedBoot (tm).
//
//####DESCRIPTIONEND####
//
//==========================================================================

#ifndef _SIMAGE_API_H_
#define _SIMAGE_API_H_

#define SIMAGE_SIG     "[(#)VtssImageTrailer(#)]"
#define SIMAGE_SIGLEN  24
#define SIMAGE_KEY     "THjwbSx!w(yShw71-ShS18%153&|91jshjdi" /* Arbitrarily chosen */
#define SIMAGE_TCOOKIE 0x64972FEB                             /* Before backpointer */

#define SIMAGE_DIGLEN        20 /* SHA1 digest len */

enum {
    SIMAGE_TLV_ARCH = 1,
    SIMAGE_TLV_CHIP,
    SIMAGE_TLV_IMGTYPE,
    SIMAGE_TLV_IMGSUBTYPE,
    SIMAGE_TLV_IMAGE,           /* (Bootloader) Image contained in Image... */
    SIMAGE_TLV_TIMESTAMP,       /* Seconds since epoch */
    SIMAGE_TLV_RSASIG,          /* RSA standard signature (public/private) */
    SIMAGE_TLV_ROTKEY,          /* RoT public key (DER) */

    /* Insert new TLV's above this line */
    SIMAGE_TLV_COUNT
};

enum {
    SIMAGE_TLV_TYPE_DWORD = 1,
    SIMAGE_TLV_TYPE_STRING,
    SIMAGE_TLV_TYPE_BINARY,
};

enum {
    SIMAGE_IMGTYPE_BOOT_LOADER = 1,
    SIMAGE_IMGTYPE_SWITCH_APP,
    SIMAGE_IMGTYPE_BOOT_LOADER_2,      // 2nd phase bootloader (RAM)
};

/* Individual TLV after parsing */
typedef struct {
    bool     valid;
    cyg_uint32      type;
    cyg_uint32      dw_value;
    const cyg_uint8 *str_ptr;
    cyg_uint32      str_len;
} simage_tlv_t;

typedef struct {
    /* Main Image data */
    const cyg_uint8 *img_ptr;
    size_t    img_len;
    /* File data - ptr as above */
    size_t    file_len;
    cyg_uint8        file_digest[SIMAGE_DIGLEN];
    /* Trailer data */
    const cyg_uint8 *trailer_ptr;
    size_t    trailer_len;
    cyg_uint8        trailer_digest[SIMAGE_DIGLEN];
    /* Parsed trailer */
    simage_tlv_t tlv[SIMAGE_TLV_COUNT];
} simage_t;

int simage_parse(simage_t *simg, const cyg_uint8 *image, size_t img_len);

bool simage_get_tlv_dword(const simage_t *simg, int id, cyg_uint32 *val);

#endif // _SIMAGE_API_H_

// ***************************************************************************
// 
//  End of file.
// 
// ***************************************************************************
