//==========================================================================
//
//      vimage.h
//
//      Support defs for VTSS linux image
//
//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
// Copyright (C) 2006 Free Software Foundation, Inc.                        
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Lars Povlsen
// Contributors: 
// Date:         2015-01-07
// Purpose:      
// Description:  
//              
// This code is part of RedBoot (tm).
//
//####DESCRIPTIONEND####
//
//==========================================================================

#ifndef _VIMAGE_H_
#define _VIMAGE_H_

#include <stddef.h>
#include "vimage_legacy.h"

#ifdef __cplusplus
extern "C" {
#endif

// Type used for 32bit unsigned integers - little endian encoded.
#if defined(CYGPKG_REDBOOT)
typedef cyg_uint32 mscc_le_u32;
typedef cyg_uint8 u8;
#else
typedef uint32_t mscc_le_u32;
#endif

#define MSCC_VIMAGE_MAGIC1    0xedd4d5de
#define MSCC_VIMAGE_MAGIC2    0x987b4c4d

typedef enum {
    MSCC_STAGE1_TLV_KERNEL      = 0,
    MSCC_STAGE1_TLV_SIGNATURE,
    MSCC_STAGE1_TLV_INITRD,
    MSCC_STAGE1_TLV_KERNEL_CMD,
    MSCC_STAGE1_TLV_METADATA,
    MSCC_STAGE1_TLV_LICENSES,
    MSCC_STAGE1_TLV_STAGE2,
} mscc_firmware_image_stage1_tlv_t;

/* Signature types */
typedef enum {
    MSCC_FIRMWARE_IMAGE_SIGNATURE_NULL   = 0,
    MSCC_FIRMWARE_IMAGE_SIGNATURE_MD5    = 1,
    MSCC_FIRMWARE_IMAGE_SIGNATURE_SHA256 = 2,
    MSCC_FIRMWARE_IMAGE_SIGNATURE_SHA512 = 3,
} mscc_firmware_image_signature_t;

typedef struct mscc_firmware_vimage {
    mscc_le_u32 magic1;         // 0xedd4d5de
    mscc_le_u32 magic2;         // 0x987b4c4d
    mscc_le_u32 version;        // 0x00000001

    mscc_le_u32 hdrlen;         // Header length
    mscc_le_u32 imglen;         // Total image length (stage1)

    char machine[32];           // Machine/board name
    char soc_name[32];          // SOC family name
    mscc_le_u32 soc_no;         // SOC family number

    mscc_le_u32 img_sign_type;  // Image signature algorithm. TLV has signature data

    // After <hdrlen> bytes;
    // struct mscc_firmware_vimage_tlv tlvs[0];
} mscc_firmware_vimage_t;

typedef struct mscc_firmware_vimage_tlv {
    mscc_le_u32 type;      // TLV type (mscc_firmware_image_stage1_tlv_t)
    mscc_le_u32 tlv_len;   // Total length of TLV (hdr, data, padding)
    mscc_le_u32 data_len;  // Data length of TLV
    u8          value[0];  // Blob data
} mscc_firmware_vimage_tlv_t;

typedef enum {
    MSCC_FIRMWARE_SIDEBAND_FILENAME,
    MSCC_FIRMWARE_SIDEBAND_STAGE2_LOCATION,
} mscc_firmware_image_sideband_tlv_t;

// Optional trailer data following firmware image in NOR
// NB: 4k byte aligned!
typedef struct mscc_firmware_sideband {
    mscc_le_u32 magic1;
    mscc_le_u32 magic2;
    u8          checksum[16];    // MD5 signature - entire trailer
    mscc_le_u32 length;
    // <mscc_firmware_vimage_tlv_t/mscc_firmware_image_sideband_tlv_t> TLvs here
} mscc_firmware_sideband_t;

int mscc_vimage_hdrcheck(mscc_firmware_vimage_t *fw, const char **errmsg);
u8 *mscc_vimage_find_tlv(const mscc_firmware_vimage_t *fw, mscc_firmware_vimage_tlv_t *tlv, mscc_firmware_image_stage1_tlv_t type);
bool mscc_vimage_validate(mscc_firmware_vimage_t *fw, mscc_firmware_vimage_tlv_t *tlv, u8 *signature);
void mscc_vimage_rot_set_invalid(void);

int isBase64Char(char);

#ifdef __cplusplus
}
#endif

#endif // _VIMAGE_H_

// ***************************************************************************
//
//  End of file.
//
// ***************************************************************************
