//==========================================================================
//
//      vimage.h
//
//      Support defs for VTSS linux image
//
//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
// Copyright (C) 2006 Free Software Foundation, Inc.                        
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Lars Povlsen
// Contributors: 
// Date:         2015-01-07
// Purpose:      
// Description:  
//              
// This code is part of RedBoot (tm).
//
//####DESCRIPTIONEND####
//
//==========================================================================

#ifndef VIMAGE_LEGACY_H
#define VIMAGE_LEGACY_H

#ifdef CYGPKG_COMPRESS_XZLIB

extern const cyg_uint8 xz_magic[6];

// Really misplaced...
extern const cyg_uint8 *linux_ramdisk_start;
extern       size_t     linux_ramdisk_size;
extern const char      *linux_command_line;

#define VIMAGE_HDR_MAGIC1 0xf7431973
#define VIMAGE_HDR_MAGIC2 0x8932ab19

// Maximum (current) length of header
#define VIMAGE_HDR_MAX (1024*1024)

// Maximum length of signature
#define VIMAGE_SHA_MAX 512

// Maximum length of kernel command line
#define VIMAGE_CMD_MAX 512

/* Signature types */
#define VIMAGE_SIG_NONE              0 // No signature used
#define VIMAGE_SIG_RSA2048_SHA256    1 // 256 bytes digest

struct firmware_vimage {
    cyg_uint32 magic1;    // 0xf7431973
    cyg_uint32 magic2;    // 0x8932ab19

    cyg_uint32 hdrlen;    // Header length
    cyg_uint32 hdrcrc;    // Header CRC
    cyg_uint32 imglen;    // Total image length;

    char machine[32];     // Machine/board name
    char soc_name[32];    // SOC family name
    cyg_uint32 soc_no;    // SOC family number

    cyg_uint32 kern_len;  // Kernel data length
    cyg_uint32 kern_off;  // Kernel data offset
    cyg_uint32 kern_crc;  // Kernel data CRC

    cyg_uint32 root_len;  // Rootfs data length
    cyg_uint32 root_off;  // Rootfs data offset
    cyg_uint32 root_crc;  // Rootfs data CRC

    cyg_uint32 meta_len;  // Metadata data length
    cyg_uint32 meta_off;  // Metadata data offset
    cyg_uint32 meta_crc;  // Metadata data CRC

    cyg_uint32 _reserved0; // Obsoleted
    cyg_uint8  _reserved1[64]; // Obsoleted

    cyg_uint32 kern_cmd_len;
    char       kern_cmd[VIMAGE_CMD_MAX]; // Kernel command line
};

#define SQUASHFS_MAGIC          0x73717368
struct squashfs_super_block {
    cyg_uint32 s_magic;
    cyg_uint32 pad0[9];
    cyg_uint32 bytes_used_lo;
    cyg_uint32 bytes_used_hi;
};

#define PAGE_SIZE               4096
static inline cyg_uint32 page_align(cyg_uint32 off, cyg_uint32 psiz)
{
    return ((off+psiz-1) & ~(psiz-1));
}

int vimage_hdrcheck(struct firmware_vimage *fw, const char **errmsg);

int xz_decompress_simple(const cyg_uint8 *src_address,
                         cyg_uint32       src_len, 
                         cyg_uint8       *dst_address,
                         cyg_uint32       dst_len,
                         cyg_uint32      *decoded_length);

const char *xz_errmsg(int ret);

static inline int vimage_is_legacy(const cyg_uint32 *image)
{
    return (image[0] == VIMAGE_HDR_MAGIC1 &&
            image[1] == VIMAGE_HDR_MAGIC2);
}

#endif    // CYGPKG_COMPRESS_XZLIB

#endif    // VIMAGE_LEGACY_H

